import gzip
import re
import tempfile
from collections import defaultdict

import numpy as np
import pandas as pd

from .. import access

# minimum number of valid samples per
# splicing event for ccle_splicing()
MIN_VALID_COUNT = 100

# minimum standard deviation per splicing
# event for ccle_splicing()
MIN_STDEV = 0.01


class Processors:
    """

    CCLE dataset processors.

    """

    @staticmethod
    def ccle_annotations(raw_path):
        """

        Process CCLE cell line annotations.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t")
        df = df.astype(str)

        return df

    @staticmethod
    def ccle_chromatin(raw_path):
        """

        Process CCLE chromatin profiling estimates.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=1)

        df = df.iloc[:, 1:]

        return df

    @staticmethod
    def ccle_translocations_svaba(raw_path):
        """

        Process CCLE SvABA calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_excel(raw_path)

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        string_cols = [
            "CCLE_name",
            "map_id",
            "bp1",
            "bp2",
            "class",
            "gene1",
            "gene2",
            "site1",
            "site2",
            "fusion",
        ]

        for col in string_cols:
            df[col] = df[col].astype(str)

        df["depmap_id"] = df["CCLE_name"].apply(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_rppa_info(raw_path):
        """

        Process CCLE RPPA antibody info.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path)
        df = df.astype(str)

        df["format_id"] = (
            df["Target_Genes"].apply(lambda x: x.replace(" ", "-"))
            + "_"
            + df["Antibody_Name"]
        )

        return df

    @staticmethod
    def ccle_rppa(raw_path):
        """

        Process CCLE RPPA values.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)

        ccle_rppa_info = access.load("ccle_rppa_info")
        antibody_name_map = dict(
            zip(ccle_rppa_info["Antibody_Name"], ccle_rppa_info["format_id"])
        )

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.columns = map(antibody_name_map.get, df.columns)
        df.index = df.index.map(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_gene_tpm(raw_path):
        """

        Process CCLE gene TPM measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)
        df = df.iloc[:, 1:]

        g19_7_definitions = access.load("g19_7_definitions")

        gene_name_map = dict(
            zip(g19_7_definitions["gene_id"], g19_7_definitions["gene_name"])
        )
        gene_name_map = defaultdict(str, gene_name_map)

        gene_names = df.index.map(lambda x: f"{gene_name_map.get(x)}_{x}")

        df = df.set_index(gene_names)
        df = np.log2(df + 1)
        df = df.astype(np.float16)

        df = df.T

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_transcript_tpm(raw_path):
        """

        Process CCLE transcript TPM measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t")

        g19_7_definitions = access.load("g19_7_definitions")

        gene_name_map = dict(
            zip(g19_7_definitions["gene_id"], g19_7_definitions["gene_name"])
        )
        gene_name_map = defaultdict(str, gene_name_map)

        df.index = df[["gene_id", "transcript_id"]].apply(
            lambda x: f"{gene_name_map.get(x['gene_id'])}_{x['transcript_id']}", axis=1
        )

        df = df.drop(["gene_id", "transcript_id"], axis=1)

        df = np.log2(df + 1)
        df = df.astype(np.float16)
        df = df.T

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_exonusage(raw_path):
        """

        Process CCLE splicing measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        def reorder_exon(exon):
            """

            Helper function for formatting exon
            splicing IDs.

            Args:
                exon (str): exon identifier

            Returns:
                Processed DataFrame

            """

            exon_split = exon.split("_")
            return "_".join(exon_split[3:]) + "_" + "_".join(exon_split[:3])

        temp = tempfile.NamedTemporaryFile(mode="wb")

        with gzip.open(raw_path, "rb") as f:

            for line in f:

                line = re.sub(b"[^\S\t\n\r]+NA\t", b"nan\t", line)
                line = re.sub(b"[^\S\t\n\r]+NA\n", b"nan\n", line)

                temp.write(line)

        df = pd.read_csv(temp.name, skiprows=2, index_col=0, sep="\t")

        temp.close()

        df.index = df.index.map(reorder_exon)

        df.index = pd.Series(df.index, index=df.index) + "_" + pd.Series(df["gene_id"])
        df = df.iloc[:, 1:]
        df = df.T

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(ccle_to_depmap.get)

        exonusage_nans = df.isna().sum(axis=0)

        keep_cols = df.columns[exonusage_nans < len(df) - MIN_VALID_COUNT]

        df = df[keep_cols]

        stdevs = df.std(axis=0)
        df = df[df.columns[stdevs >= MIN_STDEV]]

        df = df.astype(np.float16)

        return df

    @staticmethod
    def ccle_mirna(raw_path):
        """

        Process CCLE miRNA measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", skiprows=2)

        df.index = df["Description"] + "_" + df["Name"].apply(lambda x: x[1:])

        df = df.iloc[:, 2:]
        df = np.log2(df)
        df = df.T
        df = df.astype(np.float16)

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_rrbs_tss1kb(raw_path):
        """

        Process CCLE RRBS TSS 1kb measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)
        df = df.iloc[:-1, 2:]
        df = df.T

        df[df == "\tNA"] = np.nan
        df[df == "    NA"] = np.nan
        df[df == "     NA"] = np.nan
        df = df.astype(np.float16)

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_rrbs_tss_clusters(raw_path):
        """

        Process CCLE RRBS TSS cluster measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)
        df = df.iloc[:-1, 2:]
        df = df.T

        df[df == "\tNA"] = np.nan
        df[df == "    NA"] = np.nan
        df[df == "     NA"] = np.nan
        df = df.astype(np.float16)

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_rrbs_cgi_clusters(raw_path):
        """

        Process CCLE RRBS CGI cluster measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)
        df = df.iloc[:-1]

        df["cluster_pos"] = df.index
        df["cluster_n"] = df.groupby("cluster_pos").cumcount() + 1
        df.index = df["cluster_pos"].astype(str) + "-" + df["cluster_n"].astype(str)

        df = df.iloc[:, 2:-2]
        df = df.T

        df[df == "\tNA"] = np.nan
        df[df == "    NA"] = np.nan
        df[df == "     NA"] = np.nan
        df = df.astype(np.float16)

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_rrbs_enhancer_clusters(raw_path):
        """

        Process CCLE RRBS enhancer cluster measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)

        df.index = df.index + "_" + df.groupby(level=0).cumcount().astype(str)

        df = df.iloc[:, 2:]
        df.index = df.index.map(lambda x: x.replace("_", "-")) + "_enh"
        df = df.T

        df[df == "\tNA"] = np.nan
        df[df == "    NA"] = np.nan
        df[df == "     NA"] = np.nan
        df = df.astype(np.float16)

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(ccle_to_depmap.get)

        return df

    @staticmethod
    def ccle_tertp(raw_path):
        """

        Process CCLE TERT promoter calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_excel(raw_path, skiprows=4)

        df = df.set_index("depMapID")
        df["TERTp_mut"] = df["TERT_promoter_mutation"] != "wildtype"

        return df

    @staticmethod
    def ccle_msi(raw_path):
        """

        Process CCLE MSI calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_excel(raw_path, sheet_name="MSI calls")

        df = df[df["CCLE.MSI.call"].isin(["inferred-MSI", "inferred-MSS"])]

        df = df.astype(str)

        df["MSI"] = df["CCLE.MSI.call"] == "inferred-MSI"

        df = df.set_index("depMapID")

        return df

    @staticmethod
    def ccle_metabolomics(raw_path):
        """

        Process CCLE metabolomics measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path)

        df["DepMap_ID"] = df["DepMap_ID"].astype(str)

        df = df.set_index("DepMap_ID")
        df = df.drop(["CCLE_ID"], axis=1)

        df = df.astype(np.float16)

        return df

    @staticmethod
    def ccle_proteomics(raw_path):
        """

        Process CCLE proteomics measurements.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path)
        df.index = df["Gene_Symbol"].fillna("UNNAMED") + "_" + df["Uniprot_Acc"]

        df = df.iloc[:, 48:]

        df = df.T

        ccle_annotations = access.load("ccle_annotations")
        ccle_to_depmap = dict(
            zip(ccle_annotations["CCLE_ID"], ccle_annotations["depMapID"])
        )

        df.index = df.index.map(lambda x: "_".join(x.split("_")[:-1]))
        df.index = df.index.map(ccle_to_depmap.get)

        df = df[~df.index.duplicated(keep="first")]

        df = df.astype(np.float16)

        return df



from functools import reduce
from hashlib import md5
from pathlib import Path

import requests
import tqdm
from tqdm import tqdm

from .config import PROCESSED_DIR

COMPLEVEL = 9
COMPLIB = "bzip2"


class bcolors:
    HEADER = "\033[95m"
    OKBLUE = "\033[94m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"


def file_exists(file_path):
    """

    Check if a file exists.

    Args:
        file_path (str): path to the file

    """

    return Path(file_path).is_file()


def concat_cols(df, cols, delim):
    """

    Concatenate columns in a dataframe with a
    delimiter.

    Args:
        df (DataFrame): input DataFrame
        cols (list-like): columns to concatenate
        delimiter (str): delimiter to join column values

    Returns:
        Series with concatenated columns.

    """

    cols_str = [df[x].astype(str) for x in cols]

    return reduce(lambda a, b: a + delim + b, cols_str)


def parentheses_to_snake(x):
    """

    Convert a string formatted as
    "{a} ({b})" to "{a}_{b}"

    Args:
        x: input string

    Returns:
        Formatted string

    """

    x_split = x.split(" (")
    return f"{x_split[0]}_{x_split[1][:-1]}"


def export_hdf(dataset_id, df):
    """
    Exports a processed dataset to HDF.

    Args:
        dataset_id: string
            id of the dataset
        df: DataFrame
            dataset to save

    """

    df.to_hdf(
        f"{PROCESSED_DIR}/{dataset_id}.h5",
        key=dataset_id,
        complevel=COMPLEVEL,
        complib=COMPLIB,
        mode="w",
    )


def md5_match(file_path, reference_md5):
    """
    Checks if a file matches a provided md5sum

    Args:
        file_path: string
            path to the file to check
        reference_md5: string
            md5sum to check

    Returns:
        True if the file matches the md5sum, False otherwise

    """

    with open(file_path, "rb") as f:

        data = f.read()

        file_md5 = md5(data).hexdigest()

        return file_md5 == reference_md5


def download_from_url(
    url, output_path, overwrite=False, reference_md5=None, is_retry=False
):

    """
    Download a file from a URL.

    Shows progress bar and checks md5sum. Also
    checks if file is already downloaded.

    Args:
        url: string
            URL to download from
        output_path: string
            path to save the output to
        overwrite: boolean
            whether or not to overwrite the file if it already exists
        reference_md5: string
            md5sum to check
        is_retry:
            whether or not the download is a retry (if the md5sum)
            does not match, is called again

    Returns:
        True if download was successful, False otherwise

    """

    output_filename = Path(output_path).name
    output_filename_bold = f"{bcolors.BOLD}{output_filename}{bcolors.ENDC}"

    if file_exists(output_path):

        if not overwrite:

            if reference_md5 is not None:

                if not md5_match(output_path, reference_md5):

                    if not is_retry:

                        print(
                            f"{output_filename_bold} {bcolors.FAIL}does not match{bcolors.ENDC} provided md5sum. Attempting download."
                        )

                else:

                    print(
                        f"{output_filename_bold} already downloaded and {bcolors.OKGREEN}matches{bcolors.ENDC} provided md5sum."
                    )

                    return True

            else:

                print(
                    f"{output_filename_bold} already exists, skipping md5sum check (not provided)"
                )

                return True

    print(f"Downloading {url}")

    r = requests.get(url, stream=True)

    total_size = int(r.headers.get("content-length", 0))
    block_size = 1024

    t = tqdm(total=total_size, unit="iB", unit_scale=True)

    with open(output_path, "wb") as f:
        for data in r.iter_content(block_size):
            t.update(len(data))
            f.write(data)

    t.close()

    if total_size not in (0, t.n):
        print("Download error: sizes do not match")

        return False

    if reference_md5 is not None:

        if not md5_match(output_path, reference_md5):

            if not is_retry:

                print(
                    f"{output_filename_bold} {bcolors.FAIL}does not match{bcolors.ENDC} provided md5sum. Attempting second download."
                )

                download_from_url(
                    url,
                    output_path,
                    overwrite=overwrite,
                    reference_md5=reference_md5,
                    is_retry=True,
                )

            else:

                print(
                    f"{bcolors.FAIL}Second download of {output_filename_bold} failed. Recommend manual inspection.{bcolors.ENDC}"
                )

                return False

        else:

            print(
                f"{output_filename_bold} {bcolors.OKGREEN}matches{bcolors.ENDC} provided md5sum."
            )

    return True



from .checks import is_downloadable
from .config import DOWNLOAD_DIR, REFERENCE_DIR, SCHEMA
from .utils import download_from_url


def download(dataset_id):
    """

    Handler for downloading a dataset.

    Args:
        dataset_id (str): ID of the dataset

    """

    assert dataset_id in SCHEMA.index, f"{dataset_id} is not in the schema."

    dataset_row = SCHEMA.loc[dataset_id]

    dataset_type = dataset_row["type"]
    dataset_url = dataset_row["url"]
    dataset_downloaded_name = dataset_row["downloaded_name"]
    dataset_downloaded_md5 = dataset_row["downloaded_md5"]

    if dataset_type == "reference":

        download_from_url(
            dataset_url,
            f"{REFERENCE_DIR}/{dataset_downloaded_name}",
            reference_md5=dataset_downloaded_md5,
        )

    elif dataset_type == "primary_dataset":
        download_from_url(
            dataset_url,
            f"{DOWNLOAD_DIR}/{dataset_downloaded_name}",
            reference_md5=dataset_downloaded_md5,
        )

    else:

        raise ValueError("Unsupported dataset type.")


def download_all():
    """

    Download all datasets in the schema.

    """

    for _, dataset in SCHEMA.iterrows():

        dataset_id = dataset["id"]

        if is_downloadable(dataset_id):

            download(dataset_id)



from .access import description, load, schema, sources, status, summary, types
from .download import download, download_all
from .process import (
    download_and_process,
    process,
    process_all,
    remove,
    remove_all,
    remove_all_processed,
    remove_all_raw,
    remove_processed,
    remove_raw,
)

__version__ = "0.1.0"



import tempfile

import numpy as np
import pandas as pd

from .. import access
from ..utils import concat_cols, export_hdf, file_exists

# minimum number of valid samples per
# splicing event for tcga_splicing()
MIN_VALID_COUNT = 100

# minimum standard deviation per splicing
# event for tcga_splicing()
MIN_STDEV = 0.01

# tumor abbreviations map
TCGA_MAP = {
    "acute myeloid leukemia": "LAML",
    "adrenocortical cancer": "ACC",
    "bladder urothelial carcinoma": "BLCA",
    "brain lower grade glioma": "LGG",
    "breast invasive carcinoma": "BRCA",
    "cervical & endocervical cancer": "CESC",
    "cholangiocarcinoma": "CHOL",
    "colon adenocarcinoma": "COAD",
    "diffuse large B-cell lymphoma": "DLBC",
    "esophageal carcinoma": "ESCA",
    "glioblastoma multiforme": "GBM",
    "head & neck squamous cell carcinoma": "HNSC",
    "kidney chromophobe": "KICH",
    "kidney clear cell carcinoma": "KIRC",
    "kidney papillary cell carcinoma": "KIRP",
    "liver hepatocellular carcinoma": "LIHC",
    "lung adenocarcinoma": "LUAD",
    "lung squamous cell carcinoma": "LUSC",
    "mesothelioma": "MESO",
    "ovarian serous cystadenocarcinoma": "OV",
    "pancreatic adenocarcinoma": "PAAD",
    "pheochromocytoma & paraganglioma": "PCPG",
    "prostate adenocarcinoma": "PRAD",
    "rectum adenocarcinoma": "READ",
    "sarcoma": "SARC",
    "skin cutaneous melanoma": "SKCM",
    "stomach adenocarcinoma": "STAD",
    "testicular germ cell tumor": "TGCT",
    "thymoma": "THYM",
    "thyroid carcinoma": "THCA",
    "uterine carcinosarcoma": "UCS",
    "uterine corpus endometrioid carcinoma": "UCEC",
    "uveal melanoma": "UVM",
}


def tcga_splicing(raw_path, preserve_temp=False):
    """

    General handler for all TCGA splicing files.

    Args:
        raw_path (str): the complete path to the
                        raw downloaded file
        chunked (bool): whether or not to split
                        the dataset by chunks

    Returns:
        Processed DataFrame

    """

    columns = pd.read_csv(raw_path, sep="\t", index_col=0, nrows=0).columns

    dtypes = {}

    for column in columns:
        if column.startswith("TCGA-"):
            dtypes[column] = np.float16
        else:
            dtypes[column] = "str"

    chunk_iterator = pd.read_csv(
        raw_path, sep="\t", chunksize=10000, dtype=dtypes, engine="c", low_memory=False
    )

    header = True

    if preserve_temp:
        temp = open(str(raw_path) + ".csv", mode="w+")
    else:
        temp = tempfile.NamedTemporaryFile(mode="w+")

    chunk_n = 0

    for chunk in chunk_iterator:

        print(chunk_n)
        chunk_n += 1

        chunk["exon_id"] = concat_cols(
            chunk,
            [
                "gene_name",
                "event_type",
                "event_chr",
                "event_coordinates",
                "alt_region_coordinates",
            ],
            "_",
        )

        chunk = chunk.drop(
            [
                "event_id",
                "event_type",
                "event_chr",
                "event_coordinates",
                "alt_region_coordinates",
                "gene_name",
            ],
            axis=1,
        )

        chunk = chunk.set_index("exon_id")
        chunk = chunk.astype(np.float16)

        nan_counts = chunk.isna().sum(axis=1)

        keep_rows = chunk.index[nan_counts < len(chunk) - MIN_VALID_COUNT]

        chunk = chunk.filter(keep_rows, axis=0)

        stdevs = chunk.std(axis=1)

        keep_rows = chunk.index[stdevs >= MIN_STDEV]

        chunk = chunk.filter(keep_rows, axis=0)

        chunk.to_csv(temp, mode="a", header=header)

        if header:
            header = False

    columns = pd.read_csv(temp, index_col=0, nrows=0).columns
    col_dtypes = {x: np.float16 for x in columns}

    if preserve_temp:
        return columns

    merged = pd.read_csv(
        temp, index_col=0, dtype=col_dtypes, engine="c", low_memory=False
    )

    temp.close()

    merged = merged.T

    # remove prefix identifiers from names
    merged.index = merged.index.map(lambda x: x.split(".")[0])

    return merged


class Processors:
    """

    TCGA dataset processors.

    """

    @staticmethod
    def tcga_annotations(raw_path):
        """

        Process TCGA sample annotations.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)

        df["abbreviated_disease"] = df["_primary_disease"].apply(lambda x: TCGA_MAP[x])

        str_cols = ["sample_type", "_primary_disease", "abbreviated_disease"]

        for col in str_cols:

            df[col] = df[col].astype(str)

        df.index = df.index.astype(str)

        return df

    @staticmethod
    def tcga_mutations(raw_path):
        """

        Process TCGA MC3 mutation calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t")

        df = df.astype(str)
        df["start"] = df["start"].astype(int)
        df["end"] = df["end"].astype(int)

        df["DNA_VAF"] = df["DNA_VAF"].astype(np.float16)

        return df

    @staticmethod
    def tcga_msi(raw_path):
        """

        Process TCGA MSI calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_excel(raw_path)
        df = df[df["Case ID"].apply(lambda x: x[:4] == "TCGA")]
        df["sample_type"] = df["Tumor Filename"].apply(lambda x: x.split("-")[3][:-1])
        df["Case ID"] = df["Case ID"] + "-" + df["sample_type"]
        df = df.set_index("Case ID")

        return df

    @staticmethod
    def tcga_cn_continuous(raw_path):
        """

        Process TCGA continuous copy number calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)
        df = df.T
        df = df.astype(np.float16)

        return df

    @staticmethod
    def tcga_cn_thresholded(raw_path):
        """

        Process TCGA discrete (thresholded) copy number calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)
        df = df.T
        df = df.astype(np.float16)

        return df

    @staticmethod
    def tcga_cn_whitelisted(raw_path):
        """

        Process TCGA whitelisted copy number calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)
        df = df.T
        df = df.astype(np.float16)

        return df

    @staticmethod
    def tcga_normalized_gene_expression(raw_path):
        """

        Process TCGA batch effects-normalized gene expression
        estimates.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", index_col=0)

        df = df.T
        df.columns = [df.columns[i] + "_" + str(i) for i in range(len(df.columns))]

        df = df.astype(np.float16)

        return df

    @staticmethod
    def tcga_a3ss(raw_path):
        """

        Process TCGA A3SS splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return tcga_splicing(raw_path)

    @staticmethod
    def tcga_a5ss(raw_path):
        """

        Process TCGA A5SS splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return tcga_splicing(raw_path)

    @staticmethod
    def tcga_se(raw_path):
        """

        Process TCGA SE splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return tcga_splicing(raw_path)

    @staticmethod
    def tcga_ir(raw_path):
        """

        Process TCGA IR splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return tcga_splicing(raw_path)

    @staticmethod
    def tcga_mx(raw_path):
        """

        Process TCGA MX splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return tcga_splicing(raw_path, preserve_temp=True)

    @staticmethod
    def tcga_mx_chunks():
        """

        Split the TCGA MX splicing events into chunks.

        Returns:
            Chunk names

        """

        df = access.load("tcga_mx")

        cols = np.array(df.columns)

        chunk_size = 200000
        col_chunks = [
            cols[i : i + chunk_size] for i in range(0, cols.shape[0], chunk_size)
        ]

        for idx, col_chunk in enumerate(col_chunks):

            chunk = df[col_chunk]

            export_hdf(f"tcga_mx_{idx}", chunk)

        chunks_info = pd.DataFrame(index=range(8))

        return chunks_info



import pandas as pd

from .checks import is_downloadable, is_processable
from .config import DOWNLOAD_DIR, PREVIEW_DIR, PROCESSED_DIR, REFERENCE_DIR, SCHEMA
from .utils import bcolors, file_exists


def load(dataset_id, **read_hdf_kwargs):
    """

    Load a processed dataset.

    Args:
        dataset_id (str): ID of the dataset
        **kwards: additional arguments to pass to pd.read_hdf()

    """

    id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"

    dataset_path = PROCESSED_DIR / f"{dataset_id}.h5"

    assert dataset_id in SCHEMA.index, f"{id_bold} not in schema."
    assert file_exists(dataset_path), f"{id_bold} does not exist."

    df = pd.read_hdf(dataset_path, **read_hdf_kwargs)

    return df


def description(dataset_id):
    """

    Get the description of a dataset.

    Args:
        dataset_id (str): ID of the dataset

    """

    id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"

    assert dataset_id in SCHEMA.index, f"{id_bold} not in schema."

    return SCHEMA.loc[dataset_id, "description"]


def status():
    """

    Print the statuses of all datasets in the schema.

    """

    max_id_len = SCHEMA["id"].apply(len).max()

    for _, dataset in SCHEMA.iterrows():

        dataset_id = dataset["id"]
        dataset_type = dataset["type"]
        downloaded_name = dataset["downloaded_name"]

        id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"

        print(f"{id_bold}:", end="")
        print((max_id_len - len(dataset_id)) * " ", end="")

        if is_downloadable(dataset_id):

            download_path = DOWNLOAD_DIR / downloaded_name

            if dataset_type == "reference":

                download_path = REFERENCE_DIR / downloaded_name

            if file_exists(download_path):
                print(f"{bcolors.OKGREEN}downloaded{bcolors.ENDC}\t", end="")
            else:
                print(f"{bcolors.FAIL}not downloaded{bcolors.ENDC}\t", end="")

        else:

            print("\t\t", end="")

        if is_processable(dataset_id):
            if file_exists(PROCESSED_DIR / f"{dataset_id}.h5"):
                print(f"{bcolors.OKGREEN}processed{bcolors.ENDC}", end="")
            else:
                print(f"{bcolors.FAIL}not processed{bcolors.ENDC}", end="")

        print()


def summary(dataset_id):
    """

    Print the summary info of a dataset.

    Args:
        dataset_id (str): ID of the dataset

    """

    id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"

    assert dataset_id in SCHEMA.index, f"{id_bold} not in schema."

    dataset_row = SCHEMA.loc[dataset_id]

    raw_path = DOWNLOAD_DIR / dataset_row["downloaded_name"]
    processed_path = PROCESSED_DIR / f"{dataset_id}.h5"

    yes = f"{bcolors.OKGREEN}yes{bcolors.ENDC}"
    no = f"{bcolors.FAIL}no{bcolors.ENDC}"

    if dataset_row["type"] == "reference":

        dataset_summary = "\n".join(
            [
                f"{bcolors.BOLD}ID{bcolors.ENDC}: {dataset_row['id']}",
                f"{bcolors.BOLD}Type{bcolors.ENDC}: {dataset_row['type']}",
                f"{bcolors.BOLD}Description{bcolors.ENDC}: {dataset_row['description']}",
                f"{bcolors.BOLD}Source{bcolors.ENDC}: {dataset_row['source']}",
                f"{bcolors.BOLD}Source URL{bcolors.ENDC}: {dataset_row['url']}",
                f"{bcolors.BOLD}Portal URL{bcolors.ENDC}: {dataset_row['portal_url']}",
                f"{bcolors.BOLD}Size{bcolors.ENDC}: {int(dataset_row['downloaded_size'])}",
                f"{bcolors.BOLD}md5sum{bcolors.ENDC}: {dataset_row['downloaded_md5']}",
                f"{bcolors.BOLD}Raw path{bcolors.ENDC}: {DOWNLOAD_DIR}/{dataset_row['downloaded_name']}",
                f"{bcolors.BOLD}Raw exists{bcolors.ENDC}: {yes if file_exists(raw_path) else no}",
            ]
        )

    elif dataset_row["type"] == "primary_dataset":

        dataset_summary = "\n".join(
            [
                f"{bcolors.BOLD}ID{bcolors.ENDC}: {dataset_row['id']}",
                f"{bcolors.BOLD}Type{bcolors.ENDC}: {dataset_row['type']}",
                f"{bcolors.BOLD}Dependencies{bcolors.ENDC}: {dataset_row['dependencies']}",
                f"{bcolors.BOLD}Description{bcolors.ENDC}: {dataset_row['description']}",
                f"{bcolors.BOLD}Source{bcolors.ENDC}: {dataset_row['source']}",
                f"{bcolors.BOLD}Source URL{bcolors.ENDC}: {dataset_row['url']}",
                f"{bcolors.BOLD}Portal URL{bcolors.ENDC}: {dataset_row['portal_url']}",
                f"{bcolors.BOLD}Size{bcolors.ENDC}: {int(dataset_row['downloaded_size'])}",
                f"{bcolors.BOLD}md5sum{bcolors.ENDC}: {dataset_row['downloaded_md5']}",
                f"{bcolors.BOLD}Raw path{bcolors.ENDC}: {DOWNLOAD_DIR}/{dataset_row['downloaded_name']}",
                f"{bcolors.BOLD}Raw exists{bcolors.ENDC}: {yes if file_exists(raw_path) else no}",
                f"{bcolors.BOLD}Processed path{bcolors.ENDC}: {PROCESSED_DIR}/{dataset_id}",
                f"{bcolors.BOLD}Processed exists{bcolors.ENDC}: {yes if file_exists(processed_path) else no}",
            ]
        )

    elif dataset_row["type"] == "secondary_dataset":

        dataset_summary = "\n".join(
            [
                f"{bcolors.BOLD}ID{bcolors.ENDC}: {dataset_row['id']}",
                f"{bcolors.BOLD}Type{bcolors.ENDC}: {dataset_row['type']}",
                f"{bcolors.BOLD}Dependencies{bcolors.ENDC}: {dataset_row['dependencies']}",
                f"{bcolors.BOLD}Description{bcolors.ENDC}: {dataset_row['description']}",
                f"{bcolors.BOLD}Source{bcolors.ENDC}: {dataset_row['source']}",
                f"{bcolors.BOLD}Source URL{bcolors.ENDC}: {dataset_row['url']}",
                f"{bcolors.BOLD}Portal URL{bcolors.ENDC}: {dataset_row['portal_url']}",
                f"{bcolors.BOLD}Size{bcolors.ENDC}: {dataset_row['downloaded_size']}",
                f"{bcolors.BOLD}md5sum{bcolors.ENDC}: {dataset_row['downloaded_md5']}",
                f"{bcolors.BOLD}Processed path{bcolors.ENDC}: {PROCESSED_DIR}/{dataset_id}",
                f"{bcolors.BOLD}Processed exists{bcolors.ENDC}: {yes if file_exists(processed_path) else no}",
            ]
        )

    print(dataset_summary)


def schema():
    """

    Return a copy of the schema

    """

    return SCHEMA.copy(deep=True)


def types():
    """

    Return a sorted list of all the dataset types

    """

    return sorted(list(set(SCHEMA["type"])))


def sources():
    """

    Return a sorted list of all the dataset sources

    """

    print(SCHEMA["source"])

    return sorted(list(set(SCHEMA["source"])))



import os
import pathlib

import pandas as pd

"""
Contains configurations and settings used by the rest of the project.
"""

_PROJECT_DIR = pathlib.Path(__file__).resolve().parent

DATA_DIR = _PROJECT_DIR / "data"
DOWNLOAD_DIR = DATA_DIR / "raw"
REFERENCE_DIR = DATA_DIR / "reference"
PROCESSED_DIR = DATA_DIR / "processed"
PREVIEW_DIR = DATA_DIR / "preview"

SCHEMA = pd.read_csv(_PROJECT_DIR / "schema.csv")
SCHEMA.index = SCHEMA["id"]
SCHEMA["dependencies"] = SCHEMA["dependencies"].fillna("")



from .config import SCHEMA


def is_downloadable(dataset_id):
    """

    Check if a dataset is downloadable.

    Args:
        dataset_id (str): ID of the dataset

    """

    dataset_row = SCHEMA.loc[dataset_id]
    dataset_type = dataset_row["type"]

    if dataset_type in ["reference", "primary_dataset"]:

        return True

    return False


def is_processable(dataset_id):
    """

    Check if a dataset can be processed.

    Args:
        dataset_id (str): ID of the dataset

    """

    dataset_row = SCHEMA.loc[dataset_id]
    dataset_type = dataset_row["type"]

    if dataset_type in ["primary_dataset", "secondary_dataset"]:

        return True

    return False



import os
import warnings

from .access import load
from .checks import is_processable
from .config import DOWNLOAD_DIR, PREVIEW_DIR, PROCESSED_DIR, SCHEMA
from .download import download, is_downloadable
from .processors import ccle, depmap, gtex, other, tcga
from .utils import bcolors, export_hdf, file_exists


class Processors(
    ccle.Processors,
    depmap.Processors,
    gtex.Processors,
    other.Processors,
    tcga.Processors,
):
    """

    Subclass for merging the processing methods from
    the individual collections.

    """

    def __init__(self):
        return


# row to generate in generate_preview()
PREVIEW_LEN = 10


def check_dependencies(dataset_id):
    """

    Check if dataset dependencies are all met.

    Args:
        dependencies (str or NaN): comma-delimited depenencies

    """

    id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"
    assert dataset_id in SCHEMA.index, f"{id_bold} is not in the schema."

    dependencies = SCHEMA.loc[dataset_id, "dependencies"]

    if dependencies is None or dependencies != dependencies or dependencies == "":
        return

    for d in dependencies.split(","):

        d_file = f"{PROCESSED_DIR}/{d}.h5"
        d_bold = f"{bcolors.BOLD}{d}{bcolors.ENDC}"

        assert file_exists(d_file), f"Dependency {d_bold} does not exist."


def generate_preview(dataset_id):
    """

    Generate a preview of a DataFrame, saving
    to a CSV

    Args:
        dataset_id: the ID of the dataset

    """

    df = load(dataset_id, stop=PREVIEW_LEN)

    df.to_csv(f"{PREVIEW_DIR}/{dataset_id}.txt", sep="\t")


def remove_raw(dataset_id):
    """

    Remove the raw dataset file.

    Args:
        dataset_id: ID of dataset to remove

    """

    id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"

    assert dataset_id in SCHEMA.index, f"{id_bold} is not in the schema."

    dataset_row = SCHEMA.loc[dataset_id]
    downloaded_name = dataset_row["downloaded_name"]

    raw_file = f"{DOWNLOAD_DIR}/{downloaded_name}"

    if file_exists(raw_file):
        os.remove(raw_file)
    else:
        warnings.warn(f"{id_bold} is in schema, but raw file does not exist.")


def remove_processed(dataset_id):
    """

    Remove the processed dataset file.

    Args:
        dataset_id: ID of dataset to remove

    """

    id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"

    assert dataset_id in SCHEMA.index, f"{id_bold} is not in the schema."

    processed_file = f"{PROCESSED_DIR}/{dataset_id}.h5"

    if file_exists(processed_file):
        os.remove(processed_file)

        print("")
    else:
        warnings.warn(f"{id_bold} is in schema, but processed file does not exist.")


def remove(dataset_id):
    """

    Remove the raw and processed dataset files.

    Args:
        dataset_id: ID of dataset to remove

    """

    assert dataset_id in SCHEMA.index, f"{dataset_id} is not in the schema."

    remove_raw(dataset_id)
    remove_processed(dataset_id)


def remove_all_raw():
    """

    Removes all raw dataset files.

    """

    for _, dataset in SCHEMA.iterrows():

        remove_raw(dataset["id"])


def remove_all_processed():
    """

    Removes all processed dataset files.

    """

    for _, dataset in SCHEMA.iterrows():

        remove_processed(dataset["id"])


def remove_all():
    """

    Removes all raw and processed dataset files.

    """

    for _, dataset in SCHEMA.iterrows():

        remove(dataset["id"])


def process(dataset_id, overwrite=False, delete_raw=False):
    """

    Handler for processing a dataset.

    Args:
        dataset_id (str): ID of the dataset
        overwrite (bool): overwrite existing
        remove_raw (bool): remove the raw file

    """

    assert dataset_id in SCHEMA.index, f"{dataset_id} is not in the schema."

    dataset_row = SCHEMA.loc[dataset_id]

    downloaded_name = dataset_row["downloaded_name"]
    dataset_type = dataset_row["type"]

    if is_processable(dataset_id):

        output_path = f"{PROCESSED_DIR}/{dataset_id}.h5"

        id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"

        if file_exists(output_path) and not overwrite:

            print(f"{id_bold} already processed, skipping")

            return

        handler = getattr(Processors, dataset_id, None)

        if handler is not None:

            print(f"Processing {id_bold}")

            check_dependencies(dataset_id)

            if dataset_type in ["primary_dataset"]:
                df = handler(DOWNLOAD_DIR / downloaded_name)

            elif dataset_type in ["secondary_dataset"]:
                df = handler()

            export_hdf(dataset_id, df)

            generate_preview(dataset_id)

            if delete_raw:

                remove_raw(dataset_id)

        else:

            print(
                f"Handler for {id_bold} {bcolors.FAIL}not found{bcolors.ENDC}, skipping"
            )

            return

    else:
        raise ValueError("Unsupported dataset type.")


def download_and_process(dataset_id, download_kwargs={}, process_kwargs={}):
    """

    Download and process a dataset.

    Args:
        dataset_id (str): ID of the dataset
        download_kwargs (dict): arguments to pass to download()
        process_kwargs (dict): arguments to pass to process()

    """

    id_bold = f"{bcolors.BOLD}{dataset_id}{bcolors.ENDC}"

    assert dataset_id in SCHEMA.index, f"{id_bold} is not in the schema."

    if is_downloadable(dataset_id):
        download(dataset_id, **download_kwargs)
    else:
        raise ValueError("Unsupported dataset type.")

    if is_processable(dataset_id):
        process(dataset_id, **process_kwargs)
    else:
        raise ValueError("Unsupported dataset type.")


def download_and_process_all(download_kwargs={}, process_kwargs={}):
    """

    Download and process all datasets in the schema.

    """

    for _, dataset in SCHEMA.iterrows():

        download_and_process(dataset["id"], download_kwargs, process_kwargs)


def process_all(process_kwargs={}):
    """

    Process all datasets in the schema.

    """

    for _, dataset in SCHEMA.iterrows():

        process(dataset["id"], **process_kwargs)



import pandas as pd
from gtfparse import read_gtf


class Processors:
    """

    Other dataset processors.

    """

    @staticmethod
    def hg19_sizes(raw_path):
        """

        Process hg19 chromosome sizes.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t", names=["chrom", "size"])

        return df

    @staticmethod
    def g19_7_definitions(raw_path):
        """

        Process GENCODE g19 v7 gene+transcript definitions.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = read_gtf(raw_path)

        return df

    @staticmethod
    def ensembl_75_definitions(raw_path):
        """

        Process ENSEMBL v75 gene_transcript definitions.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = read_gtf(raw_path)

        return df



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cancer_data', 'cancer_data.processors']

package_data = \
{'': ['*'],
 'cancer_data': ['data/preview/.keep',
                 'data/processed/.keep',
                 'data/raw/.keep',
                 'data/reference/*']}

install_requires = \
['gtfparse>=1.2.0,<2.0.0',
 'pandas>=1.0.5,<2.0.0',
 'requests>=2.24.0,<3.0.0',
 'scipy>=1.5.1,<2.0.0',
 'tables>=3.6.1,<4.0.0',
 'tqdm>=4.48.0,<5.0.0',
 'xlrd>=1.2,<3.0']

setup_kwargs = {
    'name': 'cancer-data',
    'version': '0.3.5',
    'description': 'Preprocessing for various cancer genomics datasets',
    'long_description': '# cancer_data\n\nThis package provides unified methods for accessing popular datasets used in cancer research.\n\n**[Full documentation](https://cancer_data.kevinhu.io)**\n\n## Installation\n\n```bash\npip install cancer_data\n```\n\n## System requirements\n\nThe raw downloaded files occupy approximately 15 GB, and the processed HDFs take up about 10 GB. On a relatively recent machine with a fast SSD, processing all of the files after download takes about 3-4 hours. At least 16 GB of RAM is recommended for handling the large splicing tables.\n\n## Datasets\n\nA complete description of the datasets may be found in [schema.csv](https://github.com/kevinhu/cancer-data/blob/master/cancer_data/schema.csv).\n\n| Collection                                    | Datasets                                                                              | Portal                                                                                                                          |\n| --------------------------------------------- | ------------------------------------------------------------------------------------- | ------------------------------------------------------------------------------------------------------------------------------- |\n| Cancer Cell Line Encyclopedia (CCLE)          | Many (see portal)                                                                     | https://portals.broadinstitute.org/ccle/data (registration required)                                                            |\n| Cancer Dependency Map (DepMap)                | Genome-wide CRISPR-cas9 and RNAi screens, gene expression, mutations, and copy number | https://depmap.org/portal/download/                                                                                             |\n| The Cancer Genome Atlas (TCGA)                | Mutations, RNAseq expression and splicing, and copy number                            | https://xenabrowser.net/datapages/?cohort=TCGA%20Pan-Cancer%20(PANCAN)&removeHub=https%3A%2F%2Fxena.treehouse.gi.ucsc.edu%3A443 |\n| The Genotype-Tissue Expression (GTEx) Project | RNAseq expression and splicing                                                        | https://gtexportal.org/home/datasets                                                                                            |\n\n## Features\n\nThe goal of this package is to make statistical analysis and coordination of these datasets easier. To that end, it provides the following features:\n\n1. Harmonization: datasets within a collection have sample IDs reduced to the same format. For instance, all CCLE+DepMap datasets have been modified to use Achilles/Arxspan IDs, rather than cell line names.\n2. Speed: processed datasets are all stored in high-performance [HDF5 format](https://en.wikipedia.org/wiki/Hierarchical_Data_Format), allowing large tables to be loaded orders of magnitude faster than with CSV or TSV formats.\n3. Space: tables of purely numerical values (e.g. gene expression, methylation, drug sensitivities) are stored in half-precision format. Compression is used for all tables, resulting in size reductions by factors of over 10 for sparse matrices such as mutation tables, and over 50 for highly-redundant tables such as gene-level copy number estimates.\n\n## How it works\n\nThe [schema](https://github.com/kevinhu/cancer-data/blob/master/cancer_data/schema.csv) serves as the reference point for all datasets used. Each dataset is identified by a unique `id` column, which also serves as its access identifier.\n\nDatasets are downloaded from the location specified in `download_url`, after which they are checked against the provided `downloaded_md5` hash.\n\nThe next steps depend on the `type` of the dataset:\n\n-   `reference` datasets, such as the hg19 FASTA files, are left as-is.\n-   `primary_dataset` objects are preprocessed and converted into HDF5 format.\n-   `secondary_dataset` objects are defined as being made from `primary_dataset` objects. These are also processed and converted into HDF5 format.\n\nTo keep track of which datasets are necessary for producing another, the `dependencies` column specifies the dataset `id`s that are required for making another. For instance, the `ccle_proteomics` dataset is dependent on the `ccle_annotations` dataset for converting cell line names to Achilles IDs. When running the processing pipeline, the package will automatically check that dependencies are met, and raise an error if they are not found.\n\n## Notes\n\nSome datasets have filtering applied to reduce their size. These are listed below:\n\n-   CCLE, GTEx, and TCGA splicing datasets have been filtered to remove splicing events with many missing values as well as those with low standard deviations.\n-   When constructing binary mutation matrices (`depmap_damaging` and `depmap_hotspot`), a minimum mutation frequency is used to remove especially rare (present in less than four samples) mutations.\n-   The TCGA MX splicing dataset is extremely large (approximately 10,000 rows by 900,000 columns), so it has been split column-wise into 8 chunks.\n',
    'author': 'Kevin Hu',
    'author_email': 'kevinhuwest@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kevinhu/cancer_data',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0',
}


setup(**setup_kwargs)



from collections import Counter

import numpy as np
import pandas as pd

from .. import access
from ..utils import parentheses_to_snake

# minimum mutation frequency for depmap_damaging()
# and depmap_hotspot()
MIN_COUNT_CUTOFF = 4


class Processors:
    """

    DepMap dataset processors.

    """

    @staticmethod
    def depmap_annotations(raw_path):
        """

        Process DepMap cell line annotations.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)

        df["display_disease"] = df["lineage"].apply(
            lambda x: x.replace("_", " ").capitalize()
        )
        df["display_disease"] = df["display_disease"].apply(
            lambda x: "Unknown" if x == " " else x
        )

        df = df.astype(str)

        return df

    @staticmethod
    def avana(raw_path):
        """

        Process DepMap Avana sensitivities.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)

        df.columns = map(parentheses_to_snake, df.columns)

        df = df.astype(np.float16)

        return df

    @staticmethod
    def drive(raw_path):
        """

        Process DepMap DRIVE sensitivities.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)

        depmap_annotations = access.load("depmap_annotations")
        ccle_to_depmap = dict(
            zip(depmap_annotations["CCLE_Name"], depmap_annotations.index)
        )
        ccle_to_depmap["AZ521_STOMACH"] = "ACH-001015"
        ccle_to_depmap["GISTT1_GASTROINTESTINAL_TRACT"] = "ACH-002332"

        df.columns = map(ccle_to_depmap.get, df.columns)
        df.index = df.index.map(parentheses_to_snake)

        df.columns = df.columns.astype(str)
        df.index = df.index.astype(str)

        df = df.T
        df = df.astype(np.float16)

        return df

    @staticmethod
    def achilles(raw_path):
        """

        Process DepMap Achilles sensitivities.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)

        depmap_annotations = access.load("depmap_annotations")
        ccle_to_depmap = dict(
            zip(depmap_annotations["CCLE_Name"], depmap_annotations.index)
        )

        df.columns = map(ccle_to_depmap.get, df.columns)
        df.index = df.index.map(parentheses_to_snake)

        df.columns = df.columns.astype(str)
        df.index = df.index.astype(str)

        df = df.T
        df = df.astype(np.float16)

        return df

    @staticmethod
    def depmap_combined_rnai(raw_path):
        """

        Process combined DepMap RNAi sensitivities.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame
        """

        df = pd.read_csv(raw_path, index_col=0)

        depmap_annotations = access.load("depmap_annotations")
        ccle_to_depmap = dict(
            zip(depmap_annotations["CCLE_Name"], depmap_annotations.index)
        )
        ccle_to_depmap["AZ521_STOMACH"] = "ACH-001015"
        ccle_to_depmap["GISTT1_GASTROINTESTINAL_TRACT"] = "ACH-002332"
        ccle_to_depmap["MB157_BREAST"] = "ACH-000621"

        # SW527 presents as colorectal in depmap_annotations.
        # Cannot map to known cell line.
        df = df.drop(columns=["SW527_BREAST"])

        df.columns = map(ccle_to_depmap.get, df.columns)
        df.index = df.index.map(parentheses_to_snake)

        df.columns = df.columns.astype(str)
        df.index = df.index.astype(str)

        df = df.T
        df = df.astype(np.float16)

        return df

    @staticmethod
    def depmap_gene_tpm(raw_path):
        """

        Process DepMap gene TPM estimates.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)
        df.columns = map(parentheses_to_snake, df.columns)

        df = df.astype(np.float16)

        return df

    @staticmethod
    def depmap_mutations(raw_path):
        """

        Process DepMap mutation calls.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path)

        df = df.astype(str)

        df["Start_position"] = df["Start_position"].astype(int)
        df["End_position"] = df["End_position"].astype(int)

        df["isCOSMIChotspot"] = df["isCOSMIChotspot"].astype(bool)
        df["isTCGAhotspot"] = df["isTCGAhotspot"].astype(bool)

        return df

    @staticmethod
    def depmap_damaging():
        """

        Construct binary mutation matrix for
        DepMap damaging mutations.

        Returns:
            Processed DataFrame

        """

        df = access.load("depmap_mutations")

        df = df[df["Variant_annotation"] == "damaging"]

        # exclude rarely damaged genes
        mut_counts = Counter(df["Hugo_Symbol"])
        df["count"] = df["Hugo_Symbol"].apply(mut_counts.get)
        df = df[df["count"] >= MIN_COUNT_CUTOFF]

        # remove damaged duplicates
        df["id"] = df["Hugo_Symbol"] + "_" + df["DepMap_ID"]
        df = df.drop_duplicates(subset=["id"], keep="first")

        # dummy value for pivot table
        df["value"] = 1

        mut_mat = pd.pivot_table(
            df, values="value", index=["DepMap_ID"], columns="Hugo_Symbol", fill_value=0
        )

        mut_mat = mut_mat.astype(bool)

        return mut_mat

    @staticmethod
    def depmap_hotspot():
        """

        Construct binary mutation matrix for
        DepMap hotspot mutations.

        Returns:
            Processed DataFrame

        """

        df = access.load("depmap_mutations")

        df = df[(df[["isCOSMIChotspot", "isTCGAhotspot"]].any(1))]

        # exclude rarely damaged genes
        mut_counts = Counter(df["Hugo_Symbol"])
        df["count"] = df["Hugo_Symbol"].apply(mut_counts.get)
        df = df[df["count"] >= MIN_COUNT_CUTOFF]

        # remove hotspot duplicates
        df["id"] = df["Hugo_Symbol"] + "_" + df["DepMap_ID"]
        df = df.drop_duplicates(subset=["id"], keep="first")

        # dummy value for pivot table
        df["value"] = 1

        mut_mat = pd.pivot_table(
            df, values="value", index=["DepMap_ID"], columns="Hugo_Symbol", fill_value=0
        )

        mut_mat = mut_mat.astype(bool)

        return mut_mat

    @staticmethod
    def prism_primary_info(raw_path):
        """

        Process PRISM primary screen drug metadata.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path)

        df["format_name"] = df["name"].fillna("UNNAMED") + "_" + df["column_name"]
        df = df.astype(str)

        return df

    @staticmethod
    def prism_primary_logfold(raw_path):
        """

        Process PRISM primary screen sensitivities.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)

        prism_primary_info = access.load("prism_primary_info")
        primary_name_map = dict(
            zip(prism_primary_info["column_name"], prism_primary_info["format_name"])
        )

        df.columns = map(primary_name_map.get, df.columns)

        df = df.astype(np.float16)

        return df

    @staticmethod
    def prism_secondary_info(raw_path):
        """

        Process PRISM secondary screen drug metadata.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path)

        df["format_name"] = df["name"].fillna("UNNAMED") + "_" + df["column_name"]
        df = df.astype(str)

        return df

    @staticmethod
    def prism_secondary_logfold(raw_path):
        """

        Process PRISM secondary screen sensitivities.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)

        prism_secondary_info = access.load("prism_secondary_info")
        primary_name_map = dict(
            zip(
                prism_secondary_info["column_name"], prism_secondary_info["format_name"]
            )
        )

        df.columns = map(primary_name_map.get, df.columns)

        df = df.astype(np.float16)

        return df

    @staticmethod
    def depmap_copy_number(raw_path):
        """

        Process DepMap copy number estimates.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, index_col=0)
        df.columns = map(parentheses_to_snake, df.columns)

        df = df.astype(np.float16)

        return df

    @staticmethod
    def depmap_sanger_ceres(raw_path: str) -> pd.DataFrame:
        """
        Process Sanger CRISPR gene dependencies (CERES)
        Args:
            raw_path (str): the complete path to the
                            raw downloaded file
        Returns:
            Processed DataFrame
        """

        df = pd.read_csv(raw_path, index_col=0)

        df.columns = map(parentheses_to_snake, df.columns)

        df.index = df.index.astype(str)

        df = df.astype(np.float16)

        return df

    @staticmethod
    def depmap_sanger_chronos(raw_path: str) -> pd.DataFrame:
        """
        Process Sanger CRISPR gene dependencies (Chronos)
        Args:
            raw_path (str): the complete path to the
                            raw downloaded file
        Returns:
            Processed DataFrame
        """

        df = pd.read_csv(raw_path, index_col=0)

        df.columns = map(parentheses_to_snake, df.columns)

        df.index = df.index.astype(str)

        df = df.astype(np.float16)

        return df



import numpy as np
import pandas as pd

from .. import access
from ..utils import concat_cols

# minimum number of valid samples per
# splicing event for gtex_splicing()
MIN_VALID_COUNT = 100

# minimum standard deviation per splicing
# event for gtex_splicing()
MIN_STDEV = 0.01


def gtex_splicing(raw_path):
    """

    General handler for all GTEx splicing files.

    Args:
        raw_path (str): the complete path to the
                        raw downloaded file

    Returns:
        Processed DataFrame

    """

    df = pd.read_csv(raw_path, sep="\t")

    df["exon_id"] = concat_cols(
        df,
        [
            "gene_name",
            "event_type",
            "event_chr",
            "event_coordinates",
            "alt_region_coordinates",
        ],
        "_",
    )

    df = df.drop(
        [
            "event_id",
            "event_type",
            "event_chr",
            "event_coordinates",
            "alt_region_coordinates",
            "gene_name",
        ],
        axis=1,
    )

    df = df.set_index("exon_id")

    gtex_manifest = access.load("gtex_manifest")
    sra_to_gtex = dict(
        zip(gtex_manifest["Comment[ENA_RUN]"], gtex_manifest["Source Name"])
    )

    df.columns = [sra_to_gtex[x[:-8]] for x in df.columns]

    df = df.T
    df = df.astype(np.float16)

    nan_counts = df.isna().sum(axis=0)

    keep_cols = df.columns[nan_counts < len(df) - MIN_VALID_COUNT]

    df = df[keep_cols]

    stdevs = df.std(axis=0)
    df = df[df.columns[stdevs >= MIN_STDEV]]

    df = df.astype(np.float16)

    return df


class Processors:
    """

    GTEx dataset processors.

    """

    @staticmethod
    def gtex_2919_manifest(raw_path):
        """

        Process the GTEx 2919 manifest.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t")

        return df

    @staticmethod
    def gtex_5214_manifest(raw_path):
        """

        Process the GTEx 5214 manifest.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, sep="\t")

        return df

    @staticmethod
    def gtex_manifest():
        """

        Process the merged manifest.

        Returns:
            Processed DataFrame

        """

        gtex_manifest_1 = access.load("gtex_2919_manifest")
        gtex_manifest_2 = access.load("gtex_5214_manifest")

        gtex_manifest = pd.concat([gtex_manifest_1, gtex_manifest_2], axis=0, sort=True)

        gtex_manifest = gtex_manifest.astype(str)

        return gtex_manifest

    @staticmethod
    def gtex_gene_tpm(raw_path):
        """

        Process the merged manifest.

        Returns:
            Processed DataFrame

        """

        df = pd.read_csv(raw_path, skiprows=2, index_col=0, sep="\t")

        df.index = df["Description"] + "_" + df.index
        df.drop(["Description"], axis=1, inplace=True)
        df = df.T
        df = np.log2(df + 1)

        df = df.astype(np.float16)

        return df

    @staticmethod
    def gtex_a3ss(raw_path):
        """

        Process GTEx A3SS splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return gtex_splicing(raw_path)

    @staticmethod
    def gtex_a5ss(raw_path):
        """

        Process GTEx A5SS splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return gtex_splicing(raw_path)

    @staticmethod
    def gtex_se(raw_path):
        """

        Process GTEx SE splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return gtex_splicing(raw_path)

    @staticmethod
    def gtex_ir(raw_path):
        """

        Process GTEx IR splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return gtex_splicing(raw_path)

    @staticmethod
    def gtex_mx(raw_path):
        """

        Process GTEx MX splicing events.

        Args:
            raw_path (str): the complete path to the
                            raw downloaded file

        Returns:
            Processed DataFrame

        """

        return gtex_splicing(raw_path)
